/**
 * Copyright 2010 Jean-Noel Colin. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice, this list of
 *      conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright notice, this list
 *      of conditions and the following disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Jean-Noel Colin ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Jean-Noel Colin OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those of the
 * authors and should not be interpreted as representing official policies, either expressed
 * or implied, of Jean-Noel Colin.
 */
package be.erigo.itimesheets.beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.TimeZone;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import be.erigo.itimesheets.GlobalData;
import be.erigo.itimesheets.VisibilityConverter;
import be.erigo.itimesheets.daos.DAOException;
import be.erigo.itimesheets.daos.DAOFactory;
import be.erigo.itimesheets.spring.CustomUserDetails;
import be.erigo.itimesheets.tos.ProjectTO;
import be.erigo.itimesheets.tos.UserTO;

/**
 * 
 * @author Jean-Noel Colin
 */
@ManagedBean
@ViewScoped
public class AdminProjectBean implements Serializable {
	@ManagedProperty(value = "#{globalData}")
	private GlobalData globalData = null;

	private ProjectTO currentProject = new ProjectTO();
	private List<ProjectTO> projects = new ArrayList<ProjectTO>();
	private boolean updateMode = false;
	private ProjectTO selectedRow = null;
	private int selectedRowIndex = 0;
	private int page = 1;
	private TimeZone timezone = TimeZone.getTimeZone("CET");
	private List<UserTO> availableUsers = new ArrayList<UserTO>();
	private HashMap<Long, UserTO> availableUserIds = new HashMap<Long, UserTO>();

	private List<UserTO> selectedUsers = new ArrayList<UserTO>();
	private VisibilityConverter converter;

	public AdminProjectBean() {
		try {
			availableUsers = DAOFactory.getDAOFactory(DAOFactory.MYSQL)
					.getUserDAO().getUsers();
			for (UserTO u : availableUsers) {
				availableUserIds.put(u.getUserid(), u);
			}
		} catch (DAOException e) {
			Logger.getLogger(AdminProjectBean.class.getName()).log(
					Level.SEVERE, null, e);
		}
		loadProjects();
		converter = new VisibilityConverter();
		converter.setMap(availableUsers);
	}

	private void loadProjects() {
		try {
			projects = DAOFactory.getDAOFactory(DAOFactory.MYSQL)
					.getProjectDAO().getProjects();
		} catch (DAOException ex) {
			Logger.getLogger(AdminProjectBean.class.getName()).log(
					Level.SEVERE, null, ex);
		}
	}

	private void loadUsers() {
		try {
			if (currentProject.getId() != Integer.MAX_VALUE) {
				// Update case
				List<UserTO> selection = DAOFactory
						.getDAOFactory(DAOFactory.MYSQL).getUserDAO()
						.getProjectUsers(currentProject.getId());
				if ((selection == null) || (selection.size() == 0)) {
					selectedUsers = new ArrayList<UserTO>();
					currentProject.setVisibleToAll(true);
				} else {
					// this works
					for (UserTO u : selection) {
						selectedUsers.add(availableUserIds.get(u.getUserid()));
					}
					// this does not work
					// selectedUsers.addAll(selection);
					currentProject.setVisibleToAll(false);
				}
			} else {
				// Add case
				selectedUsers = new ArrayList<UserTO>();
				currentProject.setVisibleToAll(true);
			}
		} catch (DAOException ex) {
			Logger.getLogger(AdminProjectBean.class.getName()).log(
					Level.SEVERE, "hum", ex);
		}
	}

	public void selectAction() {
		ProjectTO selectedLine = projects.get(selectedRowIndex);
		currentProject = new ProjectTO(selectedLine);
		loadUsers();
		updateMode = true;
	}

	public String addAction() {
		long currentUserId = -1;
		if ((currentProject == null) || (currentProject.getName().equals(""))) {
			FacesContext.getCurrentInstance().addMessage(
					"project name can't be null",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"project name can't be null",
							"project name can't be null"));
			return null;
		}

		if ((currentProject.getStartDate() != null)
				&& (currentProject.getFrozenDate() != null)
				&& (currentProject.getStartDate().after(currentProject
						.getFrozenDate()))) {
			FacesContext.getCurrentInstance().addMessage(
					"frozen date must be posterior to start date",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"frozen date must be posterior to start date",
							"frozen date must be posterior to start date"));
			return null;
		}

		if ((currentProject.getEndDate() != null)
				&& (currentProject.getFrozenDate() != null)
				&& (currentProject.getFrozenDate().after(currentProject
						.getEndDate()))) {
			FacesContext.getCurrentInstance().addMessage(
					"frozen date must be anterior to end date",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"frozen date must be anterior to end date",
							"frozen date must be anterior to end date"));
			return null;
		}

		Authentication springAuthn = SecurityContextHolder.getContext()
				.getAuthentication();
		Object principal = springAuthn.getPrincipal();
		if (principal instanceof CustomUserDetails) {
			currentUserId = ((CustomUserDetails) principal).getUserId();
		} else {
			Logger.getLogger("AdminProjectBean").severe(
					"No principal found: " + principal.getClass().getName());
			FacesContext.getCurrentInstance().addMessage(
					"Invalid user in addAction",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Invalid user in addAction",
							"Invalid user in addAction"));
			return null;
		}

		System.out.println("selectedUsers: " + selectedUsers.size());

		try {
			DAOFactory.getDAOFactory(DAOFactory.MYSQL).getProjectDAO()
					.store(currentProject);
			if (!currentProject.isVisibleToAll()) {
				// Store project users
			}
		} catch (DAOException e) {
			Logger.getLogger("iTimesheets.beans").severe(
					"Error inserting project:" + e.getMessage());
			FacesContext.getCurrentInstance().addMessage(
					"Error inserting project",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Error inserting project", e.getMessage()));
		}
		currentProject = new ProjectTO();
		loadProjects();
		loadUsers();
		if (globalData != null) {
			globalData.reloadProjects();
		}
		return null;
	}

	public String updateAction() {
		long currentUserId = -1;
		if ((currentProject == null) || (currentProject.getName().equals(""))) {
			FacesContext.getCurrentInstance().addMessage(
					"project name can't be null",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"project name can't be null",
							"project name can't be null"));
			return null;
		}
		if ((currentProject.getStartDate() != null)
				&& (currentProject.getFrozenDate() != null)
				&& (currentProject.getStartDate().after(currentProject
						.getFrozenDate()))) {
			FacesContext.getCurrentInstance().addMessage(
					"frozen date must be posterior to start date",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"frozen date must be posterior to start date",
							"frozen date must be posterior to start date"));
			return null;
		}

		if ((currentProject.getEndDate() != null)
				&& (currentProject.getFrozenDate() != null)
				&& (currentProject.getFrozenDate().after(currentProject
						.getEndDate()))) {
			FacesContext.getCurrentInstance().addMessage(
					"frozen date must be anterior to end date",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"frozen date must be anterior to end date",
							"frozen date must be anterior to end date"));
			return null;
		}

		Authentication springAuthn = SecurityContextHolder.getContext()
				.getAuthentication();
		Object principal = springAuthn.getPrincipal();
		if (principal instanceof CustomUserDetails) {
			currentUserId = ((CustomUserDetails) principal).getUserId();
		} else {
			Logger.getLogger("AdminProjectBean").severe(
					"No principal found: " + principal.getClass().getName());
			FacesContext.getCurrentInstance().addMessage(
					"Invalid user in addAction",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Invalid user in addAction",
							"Invalid user in addAction"));
			return null;
		}

		System.out.println("selectedUsers: " + selectedUsers.size());

		try {
			DAOFactory.getDAOFactory(DAOFactory.MYSQL).getProjectDAO()
					.update(currentProject);
			// In case project visibility changed
			// remove all visibility records

			if (!currentProject.isVisibleToAll()) {
				// Store project users
			}
		} catch (DAOException e) {
			Logger.getLogger("iTimesheets.beans").severe(
					"Error updating project:" + e.getMessage());
			FacesContext.getCurrentInstance().addMessage(
					"Error updating project",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Error updating project", e.getMessage()));
		}
		if (globalData != null) {
			globalData.reloadProjects();
		}
		currentProject = new ProjectTO();
		updateMode = false;
		loadProjects();
		loadUsers();
		return null;
	}

	public String deleteAction() {
		long currentUserId = -1;
		if ((selectedRow == null) || (selectedRow.getName().equals(""))) {
			FacesContext.getCurrentInstance().addMessage(
					"project name can't be null",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"project name can't be null",
							"project name can't be null"));
			return null;
		}

		Authentication springAuthn = SecurityContextHolder.getContext()
				.getAuthentication();
		Object principal = springAuthn.getPrincipal();
		if (principal instanceof CustomUserDetails) {
			currentUserId = ((CustomUserDetails) principal).getUserId();
		} else {
			Logger.getLogger("AdminProjectBean").severe(
					"No principal found: " + principal.getClass().getName());
			FacesContext.getCurrentInstance().addMessage(
					"Invalid user in addAction",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Invalid user in addAction",
							"Invalid user in addAction"));
			return null;
		}
		try {
			DAOFactory.getDAOFactory(DAOFactory.MYSQL).getProjectDAO()
					.delete(selectedRow);
		} catch (DAOException e) {
			Logger.getLogger("iTimesheets.beans").severe(
					"Error deleting project:" + e.getMessage());
			FacesContext.getCurrentInstance().addMessage(
					"Error deleting project",
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Error deleting project", e.getMessage()));
		}
		if (globalData != null) {
			globalData.reloadProjects();
		}
		currentProject = new ProjectTO();
		updateMode = false;
		loadProjects();
		loadUsers();
		return null;
	}

	public String cancelAction() {
		currentProject = new ProjectTO();
		loadUsers();
		updateMode = false;
		return null;
	}

	public ProjectTO getCurrentProject() {
		return currentProject;
	}

	public void setCurrentProject(ProjectTO currentUser) {
		this.currentProject = currentUser;
	}

	public List<ProjectTO> getProjects() {
		return projects;
	}

	public void setProjects(List<ProjectTO> users) {
		this.projects = users;
	}

	public boolean getUpdateMode() {
		return updateMode;
	}

	public void setUpdateMode(boolean updateMode) {
		this.updateMode = updateMode;
	}

	/**
	 * @return the globalData
	 */
	public GlobalData getGlobalData() {
		return globalData;
	}

	/**
	 * @param globalData
	 *            the globalData to set
	 */
	public void setGlobalData(GlobalData globalData) {
		this.globalData = globalData;
	}

	/**
	 * @return the selectedRow
	 */
	public ProjectTO getSelectedRow() {
		return selectedRow;
	}

	/**
	 * @return the selectedRowIndex
	 */
	public int getSelectedRowIndex() {
		return selectedRowIndex;
	}

	/**
	 * @param selectedRow
	 *            the selectedRow to set
	 */
	public void setSelectedRow(ProjectTO selectedRow) {
		this.selectedRow = selectedRow;
	}

	/**
	 * @param selectedRowIndex
	 *            the selectedRowIndex to set
	 */
	public void setSelectedRowIndex(int selectedRowIndex) {
		this.selectedRowIndex = selectedRowIndex;
	}

	/**
	 * @return the page
	 */
	public int getPage() {
		return page;
	}

	/**
	 * @param page
	 *            the page to set
	 */
	public void setPage(int page) {
		this.page = page;
	}

	/**
	 * @return the timezone
	 */
	public TimeZone getTimezone() {
		return timezone;
	}

	/**
	 * @param timezone
	 *            the timezone to set
	 */
	public void setTimezone(TimeZone timezone) {
		this.timezone = timezone;
	}

	/**
	 * @return the availableUsers
	 */
	public List<UserTO> getAvailableUsers() {
		return availableUsers;
	}

	/**
	 * @param availableUsers
	 *            the availableUsers to set
	 */
	public void setAvailableUsers(List<UserTO> availableUsers) {
		this.availableUsers = availableUsers;
	}

	/**
	 * @return the selectedUsers
	 */
	public List<UserTO> getSelectedUsers() {
		return selectedUsers;
	}

	/**
	 * @param selectedUsers
	 *            the selectedUsers to set
	 */
	public void setSelectedUsers(List<UserTO> selectedUsers) {
		this.selectedUsers = selectedUsers;
	}

	/**
	 * @return the converter
	 */
	public VisibilityConverter getConverter() {
		return converter;
	}

	/**
	 * @param converter
	 *            the converter to set
	 */
	public void setConverter(VisibilityConverter converter) {
		this.converter = converter;
	}
}
